-------------------------------------------------------------------------------
--! @file       Keccak_F.vhd
--! @brief      Top-level of Keccak_F
--! @project    CAESAR Candidate Evaluation  
--! @author     Ekawat (ice) Homsirikamol
--! @version    1.0
--! @copyright  Copyright (c) 2014 Cryptographic Engineering Research Group
--!             ECE Department, George Mason University Fairfax, VA, U.S.A.
--!             All rights Reserved.
--! @license    This project is released under the GNU Public License.
--!             The license and distribution terms for this file may be
--!             found in the file LICENSE in this distribution or at 
--!             http://www.gnu.org/licenses/gpl-3.0.txt
--! @note       This is publicly available encryption source code that falls
--!             under the License Exception TSU (Technology and software-
--!             —unrestricted)
-------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;
use ieee.std_logic_arith.all;

use work.Keccak_pkg.all;

entity Keccak_F is
    generic (
        G_RNDS          :   integer := 24           --! Number of rounds for the permutation
    );
    port (
        --! Global signals
        clk             :   in  std_logic;
        rst             :   in  std_logic;

        --! Ctrl signals
        start           :   in  std_logic;          --! Start signal
        ready           :   out std_logic;          --! Ready signal
        done            :   out std_logic;          --! Done signal

        --! Data signals
        din             :   in  std_logic_vector(KECCAK_b-1 downto 0);
        dout            :   out std_logic_vector(KECCAK_b-1 downto 0)
    );
end Keccak_F;

-------------------------------------------------------------------------------
--! @brief  Architecture definition of Keccak_F
-------------------------------------------------------------------------------
architecture structure of Keccak_F is
    signal round            : std_logic_vector(4 downto 0);
    signal sel_in           : std_logic;
    signal en_in            : std_logic;
    
    signal din_state        : t_Keccak_state;
    signal dout_state       : t_Keccak_state;
begin

    u_map: entity work.Keccak_map(structure)
    port map (ii => din, oo => din_state);
    
    u_invmap: entity work.Keccak_invmap(structure)
    port map (ii => dout_state, oo => dout);
    
    u_dp: entity work.Keccak_F_Datapath(structure)
    port map (
        clk     => clk,
        rst     => rst,
        --! Data
        din     => din_state,
        dout    => dout_state,
        --! Controls
        round   => round,
        sel_in  => sel_in,
        en_in   => en_in);
        
    u_ctrl: entity work.Keccak_F_Control(behav)
    generic map (G_RNDS => G_RNDS)
    port map (
        clk     => clk,
        rst     => rst,
        --! External signals
        start   => start,
        ready   => ready,
        done    => done,
        --! Internal signals
        round   => round,
        sel_in  => sel_in,
        en_in   => en_in);
    
end structure;