-------------------------------------------------------------------------------
--! @file       Keccak_Round.vhd
--! @brief      Keccak's round
--! @project    CAESAR Candidate Evaluation
--! @author     Ekawat (ice) Homsirikamol
--! @version    1.0
--! @copyright  Copyright (c) 2014 Cryptographic Engineering Research Group
--!             ECE Department, George Mason University Fairfax, VA, U.S.A.
--!             All rights Reserved.
--! @license    This project is released under the GNU Public License.
--!             The license and distribution terms for this file may be
--!             found in the file LICENSE in this distribution or at 
--!             http://www.gnu.org/licenses/gpl-3.0.txt
--! @note       This is publicly available encryption source code that falls
--!             under the License Exception TSU (Technology and software-
--!             —unrestricted)
-------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

use work.Keccak_pkg.all;

entity Keccak_Round is
    port (
        rin     			: in t_Keccak_state;
        rc			   	 	: in std_logic_vector(KECCAK_WORD_SIZE-1 downto 0);
        rout    			: out t_Keccak_state
    );
end Keccak_Round;

-------------------------------------------------------------------------------
--! @brief  Basic architecture of Keccak_Round based on pseudo code
-------------------------------------------------------------------------------
architecture basic of Keccak_Round is
	function rolx ( x : std_logic_vector; y : integer) return std_logic_vector is
	begin
		return (x(x'high-y downto x'low) & x(x'high downto x'high-y+1));
	end rolx;
          
    signal aa : t_Keccak_state;    
    signal C        : t_Keccak_sheet;
    signal D        : t_Keccak_sheet;
    signal theta    : t_Keccak_state;
    signal rhopi    : t_Keccak_state;
    signal chi      : t_Keccak_state;
    signal iota     : t_Keccak_state;
begin    
    aa <= rin;
    
    gSumSheet: for x in 0 to 4 generate
        C(x) <= aa(x,0) xor aa(x,1) xor aa(x,2) xor aa(x,3) xor aa(x,4);
        D(x) <= C((x+4) mod 5) xor rolx(C((x+1) mod 5), 1);
    end generate;
        
    gRow: for y in 0 to 4 generate
        gCol: for x in 0 to 4 generate
            theta(x,              y) <= aa(x,y) xor D(x);            
            rhopi(y,(2*x+3*y) mod 5) <= rolx(theta(x,y), rotateOffset(x,y));            
            chi  (x,              y) <= rhopi(x,y) xor ((not rhopi((x+1) mod 5,y)) and rhopi((x+2) mod 5,y));
            
            gIotaExcept: if (x  = 0 and y  = 0) generate
                iota(x,y)   <= chi(x,y) xor rc;
            end generate;
            gIota:       if not (x = 0 and y = 0) generate
                iota(x,y)  <= chi(x,y);
            end generate;
        end generate;
    end generate;

    rout <= iota;
end basic;


