-------------------------------------------------------------------------------
--! @file       Keccak_F_tb.vhd
--! @brief      Testbench for Keccak_F.vhd
--! 
--! Test vectors used in this testbench is part the known answer test (KAT)
--! and the intermediate values available in the submission package of Keccak .
--!   
--! @project    CAESAR Candidate Evaluation  
--! @author     Ekawat (ice) Homsirikamol
--! @version    1.0
--! @copyright  Copyright (c) 2014 Cryptographic Engineering Research Group
--!             ECE Department, George Mason University Fairfax, VA, U.S.A.
--!             All rights Reserved.
--! @license    This project is released under the GNU Public License.
--!             The license and distribution terms for this file may be
--!             found in the file LICENSE in this distribution or at 
--!             http://www.gnu.org/licenses/gpl-3.0.txt
--! @note       This is publicly available encryption source code that falls
--!             under the License Exception TSU (Technology and software-
--!             —unrestricted)
-------------------------------------------------------------------------------
library ieee;
use ieee.std_logic_1164.all;

use work.Keccak_pkg.all;

entity Keccak_F_tb is
end Keccak_F_tb;

-------------------------------------------------------------------------------
--! @brief  Architecture definition of Keccak_F_tb
-------------------------------------------------------------------------------
architecture behav of Keccak_F_tb is
    signal rst          : std_logic := '0';
    signal clk          : std_logic := '0';
    signal start, ready, done : std_logic;
    signal di, do       : std_logic_vector(KECCAK_b-1 downto 0);
    
    signal   enClock    : boolean := TRUE;    
    constant numTVs     : integer := 4;
    constant HASH_SIZE  : integer := 256;
    constant clk_period : time    := 20 ns;    
    constant ZEROS      : std_logic_vector(KECCAK_b-1 downto 0) := (others => '0');
    type tTestVector is record
        --! Input with endianness switched.
        input           : std_logic_vector(KECCAK_b-1 downto 0);
        --! Expected output with endianness switched.
        exp_hash        : std_logic_vector(HASH_SIZE-1 downto 0);
        --! Correct output of the padded input data. The endianness of this output is swapped in a 64-bit chunk.
        act_hash        : std_logic_vector(HASH_SIZE-1 downto 0);
    end record tTestVector;
    type tTestVectorArray is array (0 to numTVs-1) of tTestVector;
    constant TV_0 : tTestVector := (
            input       => (others => '0'), 
            exp_hash    => (others => '0'),
            act_hash    => (others => '0')
        );
    constant tv : tTestVectorArray := (
        0 => (  input      => (others => '0'),
                exp_hash   => x"F1258F7940E1DDE784D5CCF933C0478AD598261EA65AA9EEBD1547306F80494D",
                act_hash   => x"E7DDE140798F25F18A47C033F9CCD584EEA95AA61E2698D54D49806F304715BD"),
        1 => (  input      => x"F1258F7940E1DDE784D5CCF933C0478AD598261EA65AA9EEBD1547306F80494D8B284E056253D057" &
                              x"FF97A42D7F8E6FD490FEE5A0A44647C48C5BDA0CD6192E76AD30A6F71B19059C30935AB7D08FFC64" &
                              x"EB5AA93F2317D635A9A6E6260D71210381A57C16DBCF555F43B831CD0347C82601F22F1A11A5569F" &
                              x"05E5635A21D9AE6164BEFEF28CC970F2613670957BC46611B87C5A554FD00ECB8C3EE88A1CCF32C8" &
                              x"940C7922AE3A26141841F924A2C509E416F53526E70465C275F644E97F30A13BEAF1FF7B5CECA249",
                exp_hash   => x"2D5C954DF96ECB3C6A332CD07057B56D093D8D1270D76B6C8A20D9B25569D094",
                act_hash   => x"3CCB6EF94D955C2D6DB55770D02C336A6C6BD770128D3D0994D06955B2D9208A"),                 
        2 => (  input      => x"00000000000001CC" & ZEROS(1599-  64 downto 1600-1088+64) &  x"8000000000000000" & ZEROS(1599-1088 downto 0),
                exp_hash   => x"560A34C7BF6DADEE88166A6944C0EDCA96566F7F6A9A54708A0B97D94BA5841E",
                act_hash   => x"EEAD6DBFC7340A56CAEDC044696A168870549A6A7F6F56961E84A54BD9970B8A"),
        3 => (  input      => x"000000000001FB41" & ZEROS(1599-  64 downto 1600-1088+64) &  x"8000000000000000" & ZEROS(1599-1088 downto 0),
                exp_hash   => x"28B3474DDACEEAA82221EAE1D95A791AE1B9BCAAF9BA07B4D2373587B717578B",
                act_hash   => x"A8EACEDA4D47B3281A795AD9E1EA2122B407BAF9AABCB9E18B5717B7873537D2"),
        others => TV_0
        );
begin
    uut:
    entity work.Keccak_F(structure)
    port map    (clk  => clk, 
                 rst  => rst,
                 start=> start, 
                 ready=> ready, 
                 done => done,
                 din  => di,   
                 dout => do);

    pClk:
    process
    begin
        while enClock = TRUE loop
            wait for clk_period/2;
            clk <= not clk;
        end loop;
        wait;
    end process;
    
    pSim:
    process    
        variable cnt_test        : integer := 0;
        variable cnt_test_failed : integer := 0;
        variable cnt_test_passed : integer := 0;
    begin
        report "    Initializing circuit...";
        rst   <= '1';
        start <= '0';
        wait for clk_period;
        rst   <= '0';
        wait for 4*clk_period;

        report "    Performing test...";
        for i in 0 to numTVs-1 loop
            --! Check test vector
            di      <= tv(i).input;
            start   <= '1';
            report "    [Start] Test number: " & integer'image(i);
            wait for clk_period;            
            di      <= (others => '0');
            start   <= '0';

            --! Verifying output of the top most 256-bit
            wait until done = '1';
            wait for clk_period*3/4;
            cnt_test := cnt_test + 1;
            if (tv(i).exp_hash = do(KECCAK_b-1 downto KECCAK_b-HASH_SIZE)) then
                report "    [Done] Test " & integer'image(i) & " |passed|.";
                cnt_test_passed := cnt_test_passed + 1;
            else
                report "    [Done] Test " & integer'image(i) & " |failed| at time " & time'image(now);
                cnt_test_failed := cnt_test_failed + 1;
            end if;
        end loop;

        wait for clk_period*5;
        enClock <= FALSE;
        report "    ===== [Done]! Test completed. =====";
        report "    [info] Total tests performed : " & integer'image(cnt_test);
        report "    [info] Number of failures    : " & integer'image(cnt_test_failed);
        report "    [info] Number of passes      : " & integer'image(cnt_test_passed);
        if (cnt_test_failed = 0) then            
            report "    [Summary] Congrats! All tests passed.";
        else
            report "    [Summary] Boohoo! Something failed. =(";
        end if;
        wait;
    end process;
    
    pCounter:
    process(clk)
        variable is_proc        : boolean;
        variable procCounter    : integer;
    begin
        --! Check start signal at the rising edge
        if rising_edge(clk) then
            if (start = '1') then
                procCounter := 0;
                is_proc     := TRUE;
            end if;
        end if;

        --! Check output status at the falling edge
        if falling_edge(clk) then
            if is_proc = TRUE then
                procCounter := procCounter + 1;
                if done = '1' then
                    is_proc := FALSE;
                    report "    [info] Processing latency = " & integer'image(procCounter);
                end if;
            end if;
        end if;
    end process;
end behav;